/*	
	This is a test program for Winford Engineering's CRD155B I/O card.
	This program uses the I/O Card API to program and control the
	I/O Card.

  Revised: October 1999
  By: JAB
  Winford Engineering
  www.winfordeng.com
*/

#include <iostream.h>
#include "wecrd.h"


int getnumber(char* prompt);//function to get a number from the user
int main_menu();//prints the main menu of options for the user

//these functions interact with the user once a
// menu option has been chosen
long opencard(); 
int configurecard(long handle); 
int outputvalue(long handle);
int inputvalue(long handle);
int readback(long handle);

int main()
{
	//In this program we only allow one card to be opened at a time.
	//This is done for simplicity and because most computers will
	//only have one I/O card in it.
	long hCard=-1;//we'll use -1 to signify that card isn't open
	
	int choice;//holds the user's menu choice
	char any;
	long result;

	cout << "-------------------------------------------------------" << endl
		 << " Test program for Winford Engineering CRD155B I/O Card"  << endl
		 << "-------------------------------------------------------" << endl;
	while(1)
	{
		choice = main_menu();
		switch(choice)
		{
		case 1://Open Card
			if(hCard > 0)//if a card is already open, then cancel the command
				cout << "An I/O card is already open." << endl;
			else
				hCard=opencard();
			break;
		case 2://Configure Card
			if(hCard < 0)//if the card hasn't been opened, then cancel the command
				cout << "You must open the card before you can configure it." << endl;
			else
				configurecard(hCard);
			break;
		case 3://Output Value
			if(hCard < 0)//if the card hasn't been opened, then cancel the command
				cout << "You must open the card before you can output a value." << endl;
			else
				outputvalue(hCard);
			break;
		case 4://Input Value
			if(hCard < 0)//if the card hasn't been opened, then cancel the command
				cout << "You must open the card before you can input a value." << endl;
			else
				inputvalue(hCard);
			break;
		case 5://Read Back
			if(hCard < 0)//if the card hasn't been opened, then cancel the command
				cout << "You must open the card before you can read back a value." << endl;
			else
				readback(hCard);
			break;
		case 6://Close Card
			if(hCard < 0)//if the card hasn't been opened, then cancel the command
				cout << "The card has not been opened." << endl;
			else
			{
				result = we_CloseCard(hCard);
				if(result == 0)//success
				{
					hCard=-1;//show that card is closed
					cout << "Card was successfully closed." << endl;
				}
				else
					cout << "Error " << result << " occured when trying to close the card." << endl;
			}
			break;
		case 7://Close All Cards (In this program, it is the same thing as Close Card, just included for example purposes)
			result = we_CloseAllCards(0);
			if(result == 0)//success
			{
				hCard=-1;//show that card is closed
				cout << "All cards were successfully closed." << endl;
			}
			else
				cout << "Error " << result << " occured when trying to close all cards." << endl;
			break;
		case 8:
			we_CloseAllCards(0);
			return(0);
			break;
		default:
			cout << "Invalid choice. Try again." << endl;
			cin.clear();
			break;
		}
		cout << "Press Enter to continue..." << endl;
		while( (any=cin.get()) != '\n' );

	}
	return(0);
	
}

int getnumber(char* prompt)
{
	int number;
	char any;//used to get rid of invalid characters

	cout << prompt;
	while((cin >> number) == 0)//loop until we get a number
	{
		cin.clear();
		while( (any=cin.get()) != '\n' );
		cout << prompt;
	}
	cin.get(any);//get rid of the newline character
	return(number);
}

int main_menu()
{//this function outputs the main menu and returns what the user types
	int choice;
	cout << "Please pick an option from the choices below:" << endl
		 << "1 - Open an I/O Card" << endl
		 << "2 - Configure Card" << endl
		 << "3 - Output a value to an output port" << endl
		 << "4 - Input a value from an input port"  << endl
		 << "5 - Read back from an output port" << endl
		 << "6 - Close the I/O Card" << endl
		 << "7 - Close all open I/O cards" << endl
		 << "8 - Exit Program" << endl << endl;

	choice = getnumber("? ");

	return(choice);
}


long opencard()
{//this function asks the user what base address the
	//card has, then attempts to open the card.
	//it returns whatever we_OpenCard returns
	unsigned short BaseAddr;
	long result;

	cout << "The valid base addresses for the CRD155B are (in decimal):" << endl
		 << "384, 388, 392, 396, 400, 404, 408, 412, 416, 420," << endl
		 << "424, 428, 432, 436, 440, 444." << endl;

	BaseAddr = getnumber("Please enter your card's base address: ");

	cout << endl;
	result = we_OpenCard(WECRD_155B, BaseAddr);
	if(result < 0)//error occurred
	{
		cout << "Error " << result << " occurred when trying to open the card." << endl;
		return(-1);//return -1 to show that the card was not successfully opened
	}
	else
	{
		cout << "Card was opened successfully." << endl;
		return(result);//return the new handle
	}
}

int configurecard(long handle)
{
	long Port;
	long Mode;
	long result;

	cout << "The CRD155B has 3 ports, so the valid ports are 1, 2, and 3." << endl;

	Port = getnumber("Please enter the port that you want to configure: ");

	cout << "The port can be put into either input or output mode:" << endl
		 << "1 = input" << endl
		 << "2 = output" << endl;

	Mode = getnumber("Please enter 1 or 2 to select the port's mode: ");

	cout << endl;
	result = we_ConfigureCard(handle, Port, Mode);
	if(result < 0)//error occurred
	{
		cout << "Error " << result << " occurred when trying to configure the card." << endl;
		return(-1);
	}
	else
	{
		cout << "Card was successfully configured" << endl;
		return(0);
	}
}

int outputvalue(long handle)
{
	long Port;
	long Value;
	long result;

	cout << "Enter the port number (1-3) that you wish to output a value to." << endl
		 << "The port should be configured for output, or else the API" << endl
		 << "will return an error" << endl;

	Port = getnumber("Enter the port number: ");

	Value = getnumber("Enter the value to output: ");

	cout << endl;
	result = we_OutputValue(handle, Port, Value);
	if(result < 0)//error occurred
	{
		cout << "Error " << result << " occurred when trying to output a value." << endl;
		return(-1);
	}
	else
	{
		cout << "The value was written to the port successfully." << endl;
		return(0);
	}
}

int inputvalue(long handle)
{
	long Port;
	long value;
	long result;

	cout << "Enter the port number (1-3) from which you wish to input a value." << endl
		 << "The port should be configured for input, or else the API" << endl
		 << "will return an error" << endl;

	Port = getnumber("Enter the port number: ");

	cout << endl;
	result = we_InputValue(handle, Port, &value);

	if(result < 0)//error occurred
	{
		cout << "Error " << result << " occurred when trying to input a value." << endl;
		return(-1);
	}
	else
	{
		cout << "A value of " << value << " was successfully read from the port." << endl;
		return(0);
	}
}

int readback(long handle)
{
	long Port;
	long value;
	long result;

	cout << "Enter the port number (1-3) from which you wish to read back a value." << endl
		 << "The port should be configured for output, or else the API" << endl
		 << "will return an error" << endl;

	Port = getnumber("Enter the port number: ");

	cout << endl;
	result = we_ReadBack(handle, Port, &value);

	if(result < 0)//error occurred
	{
		cout << "Error " << result << " occurred when trying to read back a value." << endl;
		return(-1);
	}
	else
	{
		cout << "A value of " << value << " was successfully read back from the port." << endl;
		return(0);
	}
}
